﻿Option Strict On
Option Explicit On

Imports System.Runtime.InteropServices

Public Class IMAPI2

#Region "API"
    <DllImport("Ole32.dll", EntryPoint:="CoCreateInstance")>
    Private Shared Function CoCreateInstance(<[In]> ByRef rclsid As Guid,
                                             <[In]> pUnkOuter As IntPtr,
                                             <[In]> dwClsContext As Integer,
                                             <[In]> ByRef riid As Guid,
                                             <Out> ByRef ppv As IntPtr) As Integer
    End Function

#End Region

#Region "Enums"
    Private Enum VTable_Interfaces As Integer

        ' ---=== IUnknown ===---
        ' QueryInterface = 0
        ' AddRef = 1
        ' Release = 2

        ' ---=== IDispatch ===---
        ' GetTypeInfoCount = 3
        ' GetTypeInfo = 4
        ' GetIDsOfNames = 5
        ' Invoke = 6

        ' ---=== IDiscMaster2 ===---
        GetItem = 8
        GetCount = 9
        GetIsSupportedEnvironment = 10

        ' ---=== IDiscRecorder2 ===---
        InitializeDiscRecorder = 13
        GetVendorId = 15
        GetProductId = 16
        GetProductRevision = 17
        GetVolumeName = 18
        GetVolumePathNames = 19

    End Enum
#End Region

#Region "Const"
    Private Const S_OK As Integer = &H0
    Private Const CLSCTX_INPROC_SERVER As Integer = &H1

    Private Const MsftDiscMaster2 As String = "2735412e-7f64-5b0f-8f00-5d77afbe261e"
    Private Const IID_IDiscMaster2 As String = "27354130-7f64-5b0f-8f00-5d77afbe261e"

    Private Const MsftDiscRecorder2 As String = "2735412d-7f64-5b0f-8f00-5d77afbe261e"
    Private Const IID_IDiscRecorder2 As String = "27354133-7f64-5b0f-8f00-5d77afbe261e"
#End Region

#Region "Class"
    Sub New()
        GetCdDriveInfos()
    End Sub
#End Region

#Region "Variable"
    Private ReadOnly CdDriveInfos As New List(Of CdDriveInfo)
#End Region

#Region "Structure"
    Private Structure CdDriveInfo
        Dim UniqueId As String
        Dim VendorId As String
        Dim ProductId As String
        Dim ProductRevision As String
        Dim VolumeName As String
        Dim VolumePathNames As List(Of String)
        '.... was man sonst noch so an Infos vom CD-Laufwerk haben möchte
    End Structure
#End Region

#Region "Properties"
    Public ReadOnly Property GetCdDriveCount As Integer
        Get
            Return CdDriveInfos.Count
        End Get
    End Property

    Public ReadOnly Property GetCdDriveUniqueId(index As Integer) As String
        Get
            Return CdDriveInfos(index).UniqueId
        End Get
    End Property

    Public ReadOnly Property GetCdDriveVendorId(index As Integer) As String
        Get
            Return CdDriveInfos(index).VendorId
        End Get
    End Property

    Public ReadOnly Property GetCdDriveProductId(index As Integer) As String
        Get
            Return CdDriveInfos(index).ProductId
        End Get
    End Property

    Public ReadOnly Property GetCdDriveProductRevision(index As Integer) As String
        Get
            Return CdDriveInfos(index).ProductRevision
        End Get
    End Property

    Public ReadOnly Property GetCdDriveVolumeName(index As Integer) As String
        Get
            Return CdDriveInfos(index).VolumeName
        End Get
    End Property

    Public ReadOnly Property GetCdDriveVolumePathNames(index As Integer) As List(Of String)
        Get
            Return CdDriveInfos(index).VolumePathNames
        End Get
    End Property
#End Region

#Region "Private Functions"
    Private Sub GetCdDriveInfos()
        CdDriveInfos.Clear()
        Dim pIDiscMaster2 As IntPtr = CreateInterface(MsftDiscMaster2, IID_IDiscMaster2)
        If pIDiscMaster2 <> IntPtr.Zero Then
            If DiscMaster2_GetIsSupportedEnvironment(pIDiscMaster2) Then
                Dim intCount As Integer = DiscMaster2_GetCount(pIDiscMaster2)
                If intCount > 0 Then
                    For intItem As Integer = 0 To intCount - 1
                        Dim tCdDriveInfo As New CdDriveInfo
                        Dim strUniqueID As String = DiscMaster2_GetItem(pIDiscMaster2, intItem)
                        If strUniqueID <> String.Empty Then
                            Dim pIDiscRecorder2 As IntPtr = CreateInterface(MsftDiscRecorder2, IID_IDiscRecorder2)
                            If pIDiscRecorder2 <> IntPtr.Zero Then
                                If DiscRecorder2_InitializeDiscRecorder(pIDiscRecorder2, strUniqueID) Then
                                    tCdDriveInfo.UniqueId = strUniqueID
                                    tCdDriveInfo.VendorId = DiscRecorder2_GetVendorId(pIDiscRecorder2)
                                    tCdDriveInfo.ProductId = DiscRecorder2_GetProductId(pIDiscRecorder2)
                                    tCdDriveInfo.ProductRevision = DiscRecorder2_GetProductRevision(pIDiscRecorder2)
                                    tCdDriveInfo.VolumeName = DiscRecorder2_GetVolumeName(pIDiscRecorder2)
                                    tCdDriveInfo.VolumePathNames = DiscRecorder2_GetVolumePathNames(pIDiscRecorder2)
                                    '.... was man sonst noch so an Infos vom CD-Laufwerk haben möchte
                                End If
                                Marshal.Release(pIDiscRecorder2)
                            End If
                        End If
                        CdDriveInfos.Add(tCdDriveInfo)
                    Next
                End If
            End If
            Marshal.Release(pIDiscMaster2)
        End If
    End Sub

    Private Function CreateInterface(CLSID As String, IID As String) As IntPtr
        CoCreateInstance(New Guid(CLSID), IntPtr.Zero, CLSCTX_INPROC_SERVER, New Guid(IID), CreateInterface)
    End Function

    Private Function GetMethodPtr(pInterface As IntPtr,
                                  intMethodNumber As Integer) As IntPtr
        If pInterface <> IntPtr.Zero And intMethodNumber >= 0 Then
            Dim pMethod As IntPtr() = New IntPtr(0) {}
            Marshal.Copy(IntPtr.Add(Marshal.ReadIntPtr(pInterface),
                         IntPtr.Size * intMethodNumber), pMethod, 0, 1)
            Return pMethod(0)
        End If
        Return IntPtr.Zero
    End Function
#End Region

#Region "Interface Delegates"
    Private Delegate Function IDiscMaster2_GetItem(<[In]> this As IntPtr,
                                                   <[In]> index As Integer,
                                                   <Out, MarshalAs(UnmanagedType.BStr)> ByRef value As String) As Integer

    Private Delegate Function IDiscMaster2_GetCount(<[In]> this As IntPtr,
                                                    <Out> ByRef value As Integer) As Integer

    Private Delegate Function IDiscMaster2_GetIsSupportedEnvironment(<[In]> this As IntPtr,
                                                                     <Out, MarshalAs(UnmanagedType.VariantBool)> ByRef value As Boolean) As Integer

    Private Delegate Function IDiscRecorder2_InitializeDiscRecorder(<[In]> this As IntPtr,
                                                                    <[In], MarshalAs(UnmanagedType.BStr)> recorderUniqueId As String) As Integer

    Private Delegate Function IDiscRecorder2_GetVendorId(<[In]> this As IntPtr,
                                                         <Out, MarshalAs(UnmanagedType.BStr)> ByRef value As String) As Integer

    Private Delegate Function IDiscRecorder2_GetProductId(<[In]> this As IntPtr,
                                                          <Out, MarshalAs(UnmanagedType.BStr)> ByRef value As String) As Integer

    Private Delegate Function IDiscRecorder2_GetProductRevision(<[In]> this As IntPtr,
                                                                <Out, MarshalAs(UnmanagedType.BStr)> ByRef value As String) As Integer

    Private Delegate Function IDiscRecorder2_GetVolumeName(<[In]> this As IntPtr,
                                                           <Out, MarshalAs(UnmanagedType.BStr)> ByRef value As String) As Integer

    Private Delegate Function IDiscRecorder2_GetVolumePathNames(<[In]> this As IntPtr,
                                                                <Out, MarshalAs(UnmanagedType.SafeArray, SafeArraySubType:=VarEnum.VT_VARIANT)> ByRef value As Object()) As Integer
#End Region

#Region "Interface Functions"
    Private Function DiscMaster2_GetItem(this As IntPtr, index As Integer) As String
        If this <> IntPtr.Zero Then
            Dim pMethodPtr As IntPtr = GetMethodPtr(this, VTable_Interfaces.GetItem)
            If pMethodPtr <> IntPtr.Zero Then
                Dim strValue As String = String.Empty
                If CType(Marshal.GetDelegateForFunctionPointer(pMethodPtr,
                         GetType(IDiscMaster2_GetItem)),
                         IDiscMaster2_GetItem)(this, index, strValue) = S_OK Then Return strValue


            End If
        End If
        Return String.Empty
    End Function

    Private Function DiscMaster2_GetCount(this As IntPtr) As Integer
        If this <> IntPtr.Zero Then
            Dim pMethodPtr As IntPtr = GetMethodPtr(this, VTable_Interfaces.GetCount)
            If pMethodPtr <> IntPtr.Zero Then
                Dim intValue As Integer = 0
                If CType(Marshal.GetDelegateForFunctionPointer(pMethodPtr,
                         GetType(IDiscMaster2_GetCount)),
                         IDiscMaster2_GetCount)(this, intValue) = S_OK Then Return intValue
            End If
        End If
        Return 0
    End Function

    Private Function DiscMaster2_GetIsSupportedEnvironment(this As IntPtr) As Boolean
        If this <> IntPtr.Zero Then
            Dim pMethodPtr As IntPtr = GetMethodPtr(this, VTable_Interfaces.GetIsSupportedEnvironment)
            If pMethodPtr <> IntPtr.Zero Then
                Dim bolValue As Boolean = False
                If CType(Marshal.GetDelegateForFunctionPointer(pMethodPtr,
                         GetType(IDiscMaster2_GetIsSupportedEnvironment)),
                         IDiscMaster2_GetIsSupportedEnvironment)(this, bolValue) = S_OK Then Return bolValue
            End If
        End If
        Return False
    End Function

    Private Function DiscRecorder2_InitializeDiscRecorder(this As IntPtr,
                                                          recorderUniqueId As String) As Boolean
        If this <> IntPtr.Zero Then
            Dim pMethodPtr As IntPtr = GetMethodPtr(this, VTable_Interfaces.InitializeDiscRecorder)
            If pMethodPtr <> IntPtr.Zero Then
                If CType(Marshal.GetDelegateForFunctionPointer(pMethodPtr,
                         GetType(IDiscRecorder2_InitializeDiscRecorder)),
                         IDiscRecorder2_InitializeDiscRecorder)(this, recorderUniqueId) = S_OK Then Return True
            End If
        End If
        Return False
    End Function

    Private Function DiscRecorder2_GetVendorId(this As IntPtr) As String
        If this <> IntPtr.Zero Then
            Dim pMethodPtr As IntPtr = GetMethodPtr(this, VTable_Interfaces.GetVendorId)
            If pMethodPtr <> IntPtr.Zero Then
                Dim strValue As String = String.Empty
                If CType(Marshal.GetDelegateForFunctionPointer(pMethodPtr,
                         GetType(IDiscRecorder2_GetVendorId)),
                         IDiscRecorder2_GetVendorId)(this, strValue) = S_OK Then Return strValue.Trim
            End If
        End If
        Return String.Empty
    End Function

    Private Function DiscRecorder2_GetProductId(this As IntPtr) As String
        If this <> IntPtr.Zero Then
            Dim pMethodPtr As IntPtr = GetMethodPtr(this, VTable_Interfaces.GetProductId)
            If pMethodPtr <> IntPtr.Zero Then
                Dim strValue As String = String.Empty
                If CType(Marshal.GetDelegateForFunctionPointer(pMethodPtr,
                         GetType(IDiscRecorder2_GetProductId)),
                         IDiscRecorder2_GetProductId)(this, strValue) = S_OK Then Return strValue.Trim
            End If
        End If
        Return String.Empty
    End Function

    Private Function DiscRecorder2_GetProductRevision(this As IntPtr) As String
        If this <> IntPtr.Zero Then
            Dim pMethodPtr As IntPtr = GetMethodPtr(this, VTable_Interfaces.GetProductRevision)
            If pMethodPtr <> IntPtr.Zero Then
                Dim strValue As String = String.Empty
                If CType(Marshal.GetDelegateForFunctionPointer(pMethodPtr,
                         GetType(IDiscRecorder2_GetProductRevision)),
                         IDiscRecorder2_GetProductRevision)(this, strValue) = S_OK Then Return strValue.Trim
            End If
        End If
        Return String.Empty
    End Function

    Private Function DiscRecorder2_GetVolumeName(this As IntPtr) As String
        If this <> IntPtr.Zero Then
            Dim pMethodPtr As IntPtr = GetMethodPtr(this, VTable_Interfaces.GetVolumeName)
            If pMethodPtr <> IntPtr.Zero Then
                Dim strValue As String = String.Empty
                If CType(Marshal.GetDelegateForFunctionPointer(pMethodPtr,
                         GetType(IDiscRecorder2_GetVolumeName)),
                         IDiscRecorder2_GetVolumeName)(this, strValue) = S_OK Then Return strValue.Trim
            End If
        End If
        Return String.Empty
    End Function

    Private Function DiscRecorder2_GetVolumePathNames(this As IntPtr) As List(Of String)
        Dim strList As New List(Of String)
        If this <> IntPtr.Zero Then
            Dim pMethodPtr As IntPtr = GetMethodPtr(this, VTable_Interfaces.GetVolumePathNames)
            If pMethodPtr <> IntPtr.Zero Then
                Dim objValue = New Object() {}
                If CType(Marshal.GetDelegateForFunctionPointer(pMethodPtr,
                         GetType(IDiscRecorder2_GetVolumePathNames)),
                         IDiscRecorder2_GetVolumePathNames)(this, objValue) = S_OK Then
                    Dim intCount As Integer = objValue.Length
                    If intCount > 0 Then
                        For intItem = 0 To intCount - 1
                            If objValue(intItem).GetType = GetType(String) Then
                                strList.Add(objValue(intItem).ToString.Trim)
                            End If
                        Next
                    End If
                    Return strList
                End If
            End If
        End If
        Return strList
    End Function
#End Region

End Class
